// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _StringSplaySet_h
#ifdef __GNUG__
#pragma interface
#endif
#define _StringSplaySet_h 1

#include "S.Set.h"
#include "S.SplayNode.h"

class StringSplaySet : public StringSet
{
protected:
  StringSplayNode*   root;

  StringSplayNode*   leftmost();
  StringSplayNode*   rightmost();
  StringSplayNode*   pred(StringSplayNode* t);
  StringSplayNode*   succ(StringSplayNode* t);
  void            _kill(StringSplayNode* t);
  StringSplayNode*   _copy(StringSplayNode* t);

public:
                  StringSplaySet();
                  StringSplaySet(StringSplaySet& a);
                  ~StringSplaySet();

  Pix             add(String& item);
  void            del(String& item);
  int             contains(String& item);

  void            clear();

  Pix             first();
  void            next(Pix& i);
  String&            operator () (Pix i);
  Pix             seek(String& item);

  Pix             last();
  void            prev(Pix& i);

  StringSplaySet&    operator =  (const StringSplaySet& b);
  void            operator |= (StringSplaySet& b);
  void            operator -= (StringSplaySet& b);
  void            operator &= (StringSplaySet& b);

  int             operator == (StringSplaySet& b);
  int             operator != (StringSplaySet& b);
  int             operator <= (StringSplaySet& b); 

  int             OK();
};


inline StringSplaySet::~StringSplaySet()
{
  _kill(root);
}

inline StringSplaySet::StringSplaySet()
{
  root = 0;
  count = 0;
}

inline StringSplaySet::StringSplaySet(StringSplaySet& b)
{
  count = b.count;
  root = _copy(b.root);
}


inline StringSplaySet& StringSplaySet::operator = (const StringSplaySet& b)
{
  if (this != &b)
    {
      _kill (root);
      count = b.count;
      root = _copy (b.root);
    }
  return *this;
}

inline int StringSplaySet::operator != (StringSplaySet& b)
{
  return ! (*this == b);
}

inline Pix StringSplaySet::first()
{
  return Pix(leftmost());
}

inline Pix StringSplaySet::last()
{
  return Pix(rightmost());
}

inline void StringSplaySet::next(Pix& i)
{
  if (i != 0) i = Pix(succ((StringSplayNode*)i));
}

inline void StringSplaySet::prev(Pix& i)
{
  if (i != 0) i = Pix(pred((StringSplayNode*)i));
}

inline String& StringSplaySet::operator () (Pix i)
{
  if (i == 0) error("null Pix");
  return ((StringSplayNode*)i)->item;
}

inline void StringSplaySet::clear()
{
  _kill(root);
  count = 0;
  root = 0;
}

inline int StringSplaySet::contains(String& key)
{
  return seek(key) != 0;
}

#endif
